window.Gleam.initSearch([{"doc":"pickle","title":"pickle","content":"[![Package Version](https://img.shields.io/hexpm/v/pickle)](https://hex.pm/packages/pickle)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/pickle)\n![Erlang-compatible](https://img.shields.io/badge/target-erlang-a2003e)\n![JavaScript-compatible](https://img.shields.io/badge/target-javascript-f1e05a)\n\n# Pickle 🥒\n\nA parser combinator library for Gleam that supports all targets.\n\nPickle&#39;s API does heavily rely on pipelines, thus you can create powerful parsers by chaining multiple parsers together\nwith the pipe operator.\n\nPickle also takes a different approach on its API design. In Pickle you provide an initial value to the parser (e.g., an\nempty string, list, or AST container) that&#39;s being transformed during parsing. Parsers often come with mapper\nparameters, which let you control how to transform the current value with the parsed value of the respective parser.\n\n## Demo 🥒\n\n```gleam\nimport gleam/io\nimport gleam/string\nimport pickle.{type Parser}\n\ntype Point {\n  Point(x: Int, y: Int)\n}\n\nfn new_point() -&gt; Point {\n  Point(0, 0)\n}\n\nfn point_parser() -&gt; Parser(Point, Point, String) {\n  pickle.string(&quot;(&quot;, pickle.drop)\n  |&gt; pickle.then(pickle.integer(fn(point, x) { Point(..point, x: x) }))\n  |&gt; pickle.then(pickle.string(&quot;,&quot;, pickle.drop))\n  |&gt; pickle.then(pickle.integer(fn(point, y) { Point(..point, y: y) }))\n  |&gt; pickle.then(pickle.string(&quot;)&quot;, pickle.drop))\n}\n\npub fn main() {\n  let assert Ok(point) =\n    pickle.parse(&quot;(100,-25)&quot;, new_point(), point_parser())\n\n  string.inspect(point) |&gt; io.print() // prints &quot;Point(100, -25)&quot;\n}\n```\n\n## Changelog 🥒\n\nTake a look at the [changelog](https://github.com/patrik-kuehl/pickle/blob/main/CHANGELOG.md) to get an overview of each\nrelease and its changes.\n\n## Contribution Guidelines 🥒\n\nMore information can be found [here](https://github.com/patrik-kuehl/pickle/blob/main/CONTRIBUTING.md).\n\n## License 🥒\n\nPickle is licensed under the [MIT license](https://github.com/patrik-kuehl/pickle/blob/main/LICENSE.md).\n","url":"index.html"},{"doc":"pickle","title":"pickle","content":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/), and this project adheres to\n[Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.5.0] - 2024-07-28\n\n### Added\n\n-   Added `pickle/do` to apply the initial value to a given parser.\n\n## [0.4.0] - 2024-07-26\n\n### Added\n\n-   Added `pickle/any` to parse a single token of any kind.\n-   Added `pickle/skip_until1` to skip one to `n` tokens until the terminator succeeds.\n-   Added `pickle/until1` to apply a given parser one to `n` times until the terminator succeeds.\n\n## [0.3.0] - 2024-07-20\n\n### Added\n\n-   Added `pickle/eol` to parse an end-of-line character.\n-   Added `pickle/not` to fail if the given parser succeeds.\n-   Added `pickle/lookahead` to lookahead whether the given parser succeeds.\n-   Added `pickle/skip_whitespace1` to skip one to `n` whitespace tokens.\n-   Added `pickle/whitespace1` to parse one to `n` whitespace tokens.\n\n## [0.2.0] - 2024-07-18\n\n### Added\n\n-   Added `pickle/many1` to parse tokens one to `n` times until the given parser fails.\n-   Added `pickle/uppercase_ascii_letter` to parse an uppercase ASCII letter.\n-   Added `pickle/lowercase_ascii_letter` to parse a lowercase ASCII letter.\n-   Added `pickle/ascii_letter` to parse an ASCII letter.\n\n## [0.1.0] - 2024-07-14\n\n### Added\n\n-   Added `pickle/eof` to validate that there are no further tokens to parse.\n-   Added `pickle/return` to modify the parser&#39;s value without consuming any tokens.\n-   Added `pickle/one_of` to parse tokens by trying a set of given parsers.\n-   Added `pickle/guard` to validate the value of the parser.\n-   Added `pickle/map_error` to map an error.\n-   Added `pickle/map` to map the value of the parser.\n-   Added `pickle/skip_whitespace` to skip zero to `n` whitespace tokens.\n-   Added `pickle/whitespace` to parse zero to `n` whitespace tokens.\n-   Added `pickle/float` to parse tokens as a float.\n-   Added `pickle/integer` to parse tokens as an integer.\n-   Added `pickle/octal_integer` to parse tokens as an octal integer.\n-   Added `pickle/hexadecimal_integer` to parse tokens as a hexadecimal integer.\n-   Added `pickle/decimal_integer` to parse tokens as a decimal integer.\n-   Added `pickle/binary_integer` to parse tokens as a binary integer.\n-   Added `pickle/skip_until` to skip zero to `n` tokens until the terminator succeeds.\n-   Added `pickle/until` to apply a given parser zero to `n` times until the terminator succeeds.\n-   Added `pickle/many` to parse tokens zero to `n` times until the given parser fails.\n-   Added `pickle/optional` to ignore and backtrack the parser in case the given parser fails.\n-   Added `pickle/string` to parse a specific string.\n-   Added `pickle/then` to chain parsers.\n-   Added `pickle/parse` to parse input via a given parser.\n\n[unreleased]: https://github.com/patrik-kuehl/pickle/compare/v0.5.0...HEAD\n[0.5.0]: https://github.com/patrik-kuehl/pickle/compare/v0.4.0...v0.5.0\n[0.4.0]: https://github.com/patrik-kuehl/pickle/compare/v0.3.0...v0.4.0\n[0.3.0]: https://github.com/patrik-kuehl/pickle/compare/v0.2.0...v0.3.0\n[0.2.0]: https://github.com/patrik-kuehl/pickle/compare/v0.1.0...v0.2.0\n[0.1.0]: https://github.com/patrik-kuehl/pickle/releases/tag/v0.1.0\n","url":"changelog.html"},{"doc":"pickle","title":"pickle","content":"# MIT License\n\nCopyright (c) 2024 Patrik Kühl\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated\ndocumentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the\nrights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit\npersons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the\nSoftware.\n\nTHE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE\nWARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\nCOPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR\nOTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n","url":"license.html"},{"doc":"pickle","title":"pickle","content":"","url":"#"},{"doc":"pickle","title":"pickle","content":"# Getting Started\n\nThis guide will help you getting familiar with Pickle by going through multiple challenges of increasing complexity.\n\n## Parsing Simple Structures\n\nLet&#39;s assume we&#39;re dealing with a simple format that represents a point with two axes that we receive as `(x,y)`.\n\nTo get started, let&#39;s first define a type for this.\n\n```gleam\ntype Point {\n  Point(x: Int, y: Int)\n}\n```\n\nThe format we receive is really simple. We have an opening bracket, some integer value that represents `x`, a comma,\nsome integer value that represents `y`, and a closing bracket.\n\nLet&#39;s take a look at how a parser for this can look like.\n\n```gleam\nimport pickle.{type Parser}\n\n// ...\n\nfn point_parser() -&gt; Parser(Point, Point, Nil) {\n  pickle.string(&quot;(&quot;, pickle.drop)\n  |&gt; pickle.then(pickle.integer(fn(point, x) { Point(..point, x: x) }))\n  |&gt; pickle.then(pickle.string(&quot;,&quot;, pickle.drop))\n  |&gt; pickle.then(pickle.integer(fn(point, y) { Point(..point, y: y) }))\n  |&gt; pickle.then(pickle.string(&quot;)&quot;, pickle.drop))\n}\n```\n\nThis is how parsers in Pickle look like, no matter how complex they are.\n\nThe `Parser` type has three type parameters. The first one is the type of the value we start with, the second one the\ntype of the value we end up with, and the third one is our custom error type.\n\nWe start with `pickle/string` to parse a specific string, in this case the opening bracket. Since we don&#39;t need it\neventually, we drop it via `pickle/drop`, which is a mapper provided by Pickle to drop the parsed value.\n\nWe then (no pun intended) use `pickle/then` to combine two parsers. You&#39;ll be using this a lot when using Pickle and for\nbrevity reasons, `pickle/then` won&#39;t be mentioned anymore from this point on.\n\nThe prior parser is combined with `pickle/integer` to parse an integer, our `x` value, which we use to create a new\npoint with our acquired `x` value. `pickle/integer` parses the given tokens as long as they can be represented as an\ninteger, so it doesn&#39;t expect an integer of a specific length.\n\n`pickle/integer` supports different numeric formats (binary, decimal, hexadecimal and octal). If you need or want to\nparse an integer of a specific numeric format, you can take a look at Pickle&#39;s module documentation. In this guide we&#39;ll\nonly be using decimal integers, but feel free to play around.\n\nWe then continue our adventure with `pickle/string` to parse and drop the comma.\n\nAfterwards we need to parse the `y` value of our point and as you might have guessed, use `pickle/integer` for this\nagain.\n\nLastly, we hit the jackpot by using `pickle/string` to parse and drop the closing bracket.\n\nWell done! Now we have a basic parser that we can use to parse points.\n\nTo apply the parser we use `pickle/parse`.\n\n```gleam\nimport pickle.{type Parser}\n\n/// ...\n\nfn new_point() -&gt; Point {\n  Point(0, 0)\n}\n\npub fn main() {\n  let assert Ok(point) =\n    pickle.parse(&quot;(20,10)&quot;, new_point(), point_parser())\n\n  string.inspect(point) |&gt; io.print() // prints &quot;Point(20, 10)&quot;\n}\n```\n\n`pickle/parse` takes three arguments. The first one is the input string, the second one the initial value, and the third\none the parser to apply. The initial value doesn&#39;t have to be a simple data structure. When parsing DSLs or even\nprogramming languages you most probably want to use a custom AST type to initialize the parser value with.\n\n## Parsing Variants\n\nLet&#39;s add some seasoning to our problem domain here. Our point can now come in different shapes, `(x,y)` and `[x,y]`.\n\nBefore you write two parsers with a lot of duplication for each shape, take a look at the following parsers and compare\nit to the prior one we&#39;ve written.\n\n```gleam\nimport pickle.{type Parser}\n\n// ...\n\nfn do_point_parser(\n  opening_bracket: String,\n  closing_bracket: String,\n) -&gt; Parser(Point, Point, Nil) {\n  pickle.string(opening_bracket, pickle.drop)\n  |&gt; pickle.then(pickle.integer(fn(point, x) { Point(..point, x: x) }))\n  |&gt; pickle.then(pickle.string(&quot;,&quot;, pickle.drop))\n  |&gt; pickle.then(pickle.integer(fn(point, y) { Point(..point, y: y) }))\n  |&gt; pickle.then(pickle.string(closing_bracket, pickle.drop))\n}\n\nfn point_parser() -&gt; Parser(Point, Point, Nil) {\n  pickle.one_of([do_point_parser(&quot;(&quot;, &quot;)&quot;), do_point_parser(&quot;[&quot;, &quot;]&quot;)])\n}\n```\n\nWe handle both kinds of brackets by using `pickle/one_of`, which takes zero to `n` parsers to try in order, and in this\ncase we feed it with two parsers by using our parameterized `do_point_parser` function, enabling us to specify different\nopening and closing brackets.\n\nLet&#39;s see it in action.\n\n```gleam\nimport pickle.{type Parser}\n\n/// ...\n\npub fn main() {\n  let assert Ok(first_point) =\n    pickle.parse(&quot;(20,-5)&quot;, new_point(), point_parser())\n\n  let assert Ok(second_point) =\n    pickle.parse(&quot;[10,325]&quot;, new_point(), point_parser())\n\n  string.inspect(first_point) |&gt; io.print() // prints &quot;Point(20, -5)&quot;\n  string.inspect(second_point) |&gt; io.print() // prints &quot;Point(10, 325)&quot;\n}\n```\n\n## Perform Validation\n\nPickle offers the possibility to validate the value of the parser. To showcase this, a new requirement has been\ndelivered by our fellow UPS driver.\n\nThe `x` and `y` values of our point cannot be less than -10 and greater than 10. Why? Why not.\n\nLet&#39;s first define a custom error type for our validation purposes to reflect this requirement.\n\n```gleam\ntype PointAxis {\n  X\n  Y\n}\n\ntype PointError {\n  ValueIsLessThanMinusTen(axis: PointAxis)\n  ValueIsGreaterThanTen(axis: PointAxis)\n}\n```\n\nWe then need to replace the third type parameter of `Parser` with our custom error type to tell Pickle what error type\nto expect in case of a validation failure.\n\n```gleam\nimport pickle.{type Parser}\n\n// ...\n\nfn do_point_parser(\n  opening_bracket: String,\n  closing_bracket: String,\n) -&gt; Parser(Point, Point, PointError) {\n  pickle.string(opening_bracket, pickle.drop)\n  |&gt; pickle.then(pickle.integer(fn(point, x) { Point(..point, x: x) }))\n  |&gt; pickle.then(pickle.string(&quot;,&quot;, pickle.drop))\n  |&gt; pickle.then(pickle.integer(fn(point, y) { Point(..point, y: y) }))\n  |&gt; pickle.then(pickle.string(closing_bracket, pickle.drop))\n}\n\nfn point_parser() -&gt; Parser(Point, Point, PointError) {\n  pickle.one_of([do_point_parser(&quot;(&quot;, &quot;)&quot;), do_point_parser(&quot;[&quot;, &quot;]&quot;)])\n}\n```\n\nNow we need to add some validation. For this purpose we use `pickle/guard`.\n\n```gleam\nimport pickle.{type Parser}\n\n// ...\n\nfn validate_x_value() -&gt; Parser(Point, Point, PointError) {\n  pickle.guard(fn(point: Point) { point.x &gt;= -10 }, ValueIsLessThanMinusTen(X))\n  |&gt; pickle.then(pickle.guard(\n    fn(point: Point) { point.x &lt;= 10 },\n    ValueIsGreaterThanTen(X),\n  ))\n}\n\nfn validate_y_value() -&gt; Parser(Point, Point, PointError) {\n  pickle.guard(fn(point: Point) { point.y &gt;= -10 }, ValueIsLessThanMinusTen(Y))\n  |&gt; pickle.then(pickle.guard(\n    fn(point: Point) { point.y &lt;= 10 },\n    ValueIsGreaterThanTen(Y),\n  ))\n}\n\nfn do_point_parser(\n  opening_bracket: String,\n  closing_bracket: String,\n) -&gt; Parser(Point, Point, PointError) {\n  pickle.string(opening_bracket, pickle.drop)\n  |&gt; pickle.then(pickle.integer(fn(point, x) { Point(..point, x: x) }))\n  |&gt; pickle.then(pickle.string(&quot;,&quot;, pickle.drop))\n  |&gt; pickle.then(pickle.integer(fn(point, y) { Point(..point, y: y) }))\n  |&gt; pickle.then(pickle.string(closing_bracket, pickle.drop))\n}\n\nfn point_parser() -&gt; Parser(Point, Point, PointError) {\n  pickle.one_of([do_point_parser(&quot;(&quot;, &quot;)&quot;), do_point_parser(&quot;[&quot;, &quot;]&quot;)])\n  |&gt; pickle.then(validate_x_value())\n  |&gt; pickle.then(validate_y_value())\n}\n```\n\nWe could certainly reduce the duplication in this validation logic and replace these magic numbers with constants, but\nthat&#39;s not the focus here.\n\nFrom now on we&#39;ve got a parser with validation logic to ensure our points cannot have `x` and `y` values that are less\nthan -10 and greater than 10.\n\nTrying to parse a point with invalid values now results in a `GuardError`, which contains our error value.\n\n```gleam\nimport pickle.{type Parser, GuardError}\n\n/// ...\n\npub fn main() {\n  let assert Ok(point) =\n    pickle.parse(&quot;(20,-5)&quot;, new_point(), point_parser())\n\n  let assert Error(GuardError(error)) =\n    pickle.parse(&quot;[10,325]&quot;, new_point(), point_parser())\n\n  string.inspect(first_point) |&gt; io.print() // prints &quot;Point(20, -5)&quot;\n  string.inspect(error) |&gt; io.print() // prints &quot;ValueIsGreaterThanTen(Y)&quot;\n}\n```\n\nPickle not only returns errors when some validation failed, but also when some of the parsers failed to parse the input.\nYou should keep in mind that the `GuardError` type is exclusive to validation-specific failures.\n\n## Parsing Sequences\n\nFine, we&#39;re able to parse a point, but what about a list of points?\n\nLet&#39;s assume the receive a list of points as a comma-separated list (e.g., `(2,-4),(10,0),[-5,6]`).\n\nPickle happens to offer just the right tool for this job, `pickle/many`. This parser applies the given parser zero to\n`n` times until it fails and is offering us a way to accumulate the collected points.\n\n```gleam\nimport pickle.{type Parser}\n\n// ...\n\nfn points_parser() -&gt; Parser(List(Point), List(Point), PointError) {\n  pickle.many(\n    new_point(),\n    point_parser()\n      |&gt; pickle.then(\n        pickle.one_of([pickle.string(&quot;,&quot;, pickle.drop), pickle.eof()]),\n      ),\n    pickle.prepend_to_list,\n  )\n}\n```\n\nHere we use our `point_parser` function combined with a parser to either parse a comma or EOF to set the head of the\nparser to the next point. `pickle/many` runs our parser zero to `n` times until it fails. Each parser will be given a\nblank point as an initial value. Afterwards we prepend the parsed point to our list of points via\n`pickle/prepend_to_list`, which is another mapper provided by Pickle.\n\nKeep in mind that `pickle/many` never fails and adheres to the best-effort error handling strategy. As soon as it\nencounters invalid input it just stops consuming any more tokens and returns the collected items that could be parsed\nuntil the point of failure.\n\nThis means that you could end up with no collected points (an empty list) because you provided invalid input to the\nparser.\n\nIf the given parser fails due to some validation constraint, moving this validation logic outside of `pickle/many` might\nbe a viable option, so you&#39;re still able to convey validation issues to the consumer while letting `pickle/many` collect\nitems with an invalid state before running the validation. The best approach depends on your use case eventually.\n\n```gleam\nimport pickle.{type Parser}\n\n/// ...\n\npub fn main() {\n  let assert Ok(points) =\n    pickle.parse(&quot;(20,-5),[0,10]&quot;, [], points_parser())\n\n  let assert Ok(points2) =\n    pickle.parse(&quot;(20,-5),gibberish&quot;, [], points_parser())\n\n  let assert Ok(nothing) =\n    pickle.parse(&quot;[50,-50],(-100,25)&quot;, [], points_parser())\n\n  let assert Ok(nothing2) =\n    pickle.parse(&quot;gibberish&quot;, [], points_parser())\n\n  string.inspect(points) |&gt; io.print() // prints &quot;[Point(0, 10), Point(20, -5)]&quot;\n  string.inspect(points2) |&gt; io.print() // prints &quot;[Point(20, -5)]&quot;\n  string.inspect(nothing) |&gt; io.print() // prints &quot;[]&quot;\n  string.inspect(nothing2) |&gt; io.print() // prints &quot;[]&quot;\n}\n```\n\n## You&#39;ve Made It!\n\nCongratulations! You&#39;ve finished the getting started guide and learned about the fundamentals of Pickle. Happy parsing!\n\nThe tested final implementation of this parser can be found in `test/examples/point_test.gleam`.\n\n## Additional Challenges\n\nYou could think about adding further shapes like `{x,y}`, or add support for another delimiter like a semicolon.\n\nOne thing to keep in mind is that Pickle is scannerless, thus there&#39;s no separate lexer to tokenize the input. This\nmeans that the parser covers responsibilities usually taken care of by a lexer like handling whitespace. As of now, our\npoint parser cannot handle input with whitespace sprinkled in.\n\nThe parser will fail if we provide input like `(x, y)`, or `( x,y )`.\n\nYou could extend the parser to handle whitespace, in this case by ignoring it. For this you can use\n`pickle/skip_whitespace`.\n","url":"docs/getting-started.html"},{"doc":"pickle","title":"ExpectedToken","content":"pub type ExpectedToken {\n  Eol\n  Eof\n  NonEof\n  Float\n  Integer\n  OctalDigit\n  BinaryDigit\n  DecimalDigit\n  HexadecimalDigit\n  DecimalDigitOrPoint\n  AsciiLetter\n  LowercaseAsciiLetter\n  UppercaseAsciiLetter\n  Whitespace\n  String(String)\n}\n The type to represent a kind of token that was expected at a\n specific position of the input that couldn&#39;t be found.\nEol\n\n\nEof\n\n\nNonEof\n\n\nFloat\n\n\nInteger\n\n\nOctalDigit\n\n\nBinaryDigit\n\n\nDecimalDigit\n\n\nHexadecimalDigit\n\n\nDecimalDigitOrPoint\n\n\nAsciiLetter\n\n\nLowercaseAsciiLetter\n\n\nUppercaseAsciiLetter\n\n\nWhitespace\n\n\nString(String)\n\n\nSynonyms:\npickle.ExpectedToken\npickle ExpectedToken","url":"pickle.html#ExpectedToken"},{"doc":"pickle","title":"Parsed","content":"pub opaque type Parsed(a)\n The type to store unconsumed tokens, the position of the\n parser and the current value.\n\nSynonyms:\npickle.Parsed\npickle Parsed","url":"pickle.html#Parsed"},{"doc":"pickle","title":"Parser","content":"pub type Parser(a, b, c) =\n  fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c))\n The parser type that is an alias for a function that is\n responsible for parsing input.\n\nSynonyms:\npickle.Parser\npickle Parser","url":"pickle.html#Parser"},{"doc":"pickle","title":"ParserFailure","content":"pub type ParserFailure(a) {\n  UnexpectedToken(\n    expected_token: ExpectedToken,\n    actual_token: String,\n    pos: ParserPosition,\n  )\n  UnexpectedEof(\n    expected_token: ExpectedToken,\n    pos: ParserPosition,\n  )\n  OneOfError(failures: List(ParserFailure(a)))\n  GuardError(error: a, pos: ParserPosition)\n  Until1Error(pos: ParserPosition)\n  NotError(error: a, pos: ParserPosition)\n  CustomError(error: a)\n}\n The error type to represent a reason why a parser failed to\n parse the input.\nUnexpectedToken(\n  expected_token: ExpectedToken,\n  actual_token: String,\n  pos: ParserPosition,\n)\n\n\nUnexpectedEof(expected_token: ExpectedToken, pos: ParserPosition)\n\n\nOneOfError(failures: List(ParserFailure(a)))\n\n\nGuardError(error: a, pos: ParserPosition)\n\n\nUntil1Error(pos: ParserPosition)\n\n\nNotError(error: a, pos: ParserPosition)\n\n\nCustomError(error: a)\n\n\nSynonyms:\npickle.ParserFailure\npickle ParserFailure","url":"pickle.html#ParserFailure"},{"doc":"pickle","title":"ParserPosition","content":"pub type ParserPosition {\n  ParserPosition(row: Int, col: Int)\n}\n The type to represent the current position of the parser.\nParserPosition(row: Int, col: Int)\n\n\nSynonyms:\npickle.ParserPosition\npickle ParserPosition","url":"pickle.html#ParserPosition"},{"doc":"pickle","title":"any","content":"pub fn any(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a single token of any kind and fails if there is no further input\n left to parse.\nSynonyms:\npickle.any\npickle any","url":"pickle.html#any"},{"doc":"pickle","title":"apppend_to_string","content":"pub fn apppend_to_string(\n  value: String,\n  appendage: String,\n) -&gt; String\n A mapper to append the parsed string of the child parser to\n the string value of the parent parser.\nSynonyms:\npickle.apppend_to_string\npickle apppend_to_string","url":"pickle.html#apppend_to_string"},{"doc":"pickle","title":"ascii_letter","content":"pub fn ascii_letter(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses an ASCII letter.\nSynonyms:\npickle.ascii_letter\npickle ascii_letter","url":"pickle.html#ascii_letter"},{"doc":"pickle","title":"binary_integer","content":"pub fn binary_integer(\n  mapper: fn(a, Int) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a binary integer.\nSynonyms:\npickle.binary_integer\npickle binary_integer","url":"pickle.html#binary_integer"},{"doc":"pickle","title":"decimal_integer","content":"pub fn decimal_integer(\n  mapper: fn(a, Int) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a decimal integer.\nSynonyms:\npickle.decimal_integer\npickle decimal_integer","url":"pickle.html#decimal_integer"},{"doc":"pickle","title":"do","content":"pub fn do(\n  initial_value: a,\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n  mapper: fn(d, b) -&gt; d,\n) -&gt; fn(Parsed(d)) -&gt; Result(Parsed(d), ParserFailure(c))\n Applies the initial value to the given parser.\n \n This parser is especially useful when you want or need to transform a\n value of a different type than the one the parent parser holds\n (e.g., a type that the AST type of the parent parser can contain).\nSynonyms:\npickle.do\npickle do","url":"pickle.html#do"},{"doc":"pickle","title":"drop","content":"pub fn drop(value: a, : b) -&gt; a\n A mapper to drop the parsed value of the child parser, thus\n leaving the value of the parent parser unchanged.\nSynonyms:\npickle.drop\npickle drop","url":"pickle.html#drop"},{"doc":"pickle","title":"eof","content":"pub fn eof() -&gt; fn(Parsed(a)) -&gt;\n  Result(Parsed(a), ParserFailure(b))\n Parses successfully when there is no further input left to parse. \nSynonyms:\npickle.eof\npickle eof","url":"pickle.html#eof"},{"doc":"pickle","title":"eol","content":"pub fn eol(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses an end-of-line character.\nSynonyms:\npickle.eol\npickle eol","url":"pickle.html#eol"},{"doc":"pickle","title":"float","content":"pub fn float(\n  mapper: fn(a, Float) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a decimal float.\n \n This function will be adjusted to support different numeric\n formats in later versions.\nSynonyms:\npickle.float\npickle float","url":"pickle.html#float"},{"doc":"pickle","title":"guard","content":"pub fn guard(\n  predicate: fn(a) -&gt; Bool,\n  error: b,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Validates the value of the parser.\n \n If the validation fails, a `GuardError` with the given `error`\n will be returned.\nSynonyms:\npickle.guard\npickle guard","url":"pickle.html#guard"},{"doc":"pickle","title":"hexadecimal_integer","content":"pub fn hexadecimal_integer(\n  mapper: fn(a, Int) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a hexadecimal integer.\nSynonyms:\npickle.hexadecimal_integer\npickle hexadecimal_integer","url":"pickle.html#hexadecimal_integer"},{"doc":"pickle","title":"integer","content":"pub fn integer(\n  mapper: fn(a, Int) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses an integer of different numeric formats (binary, decimal, hexadecimal\n and octal).\nSynonyms:\npickle.integer\npickle integer","url":"pickle.html#integer"},{"doc":"pickle","title":"lookahead","content":"pub fn lookahead(\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(c))\n Looksahead whether the given parser succeeds and backtracks if\n it does.\nSynonyms:\npickle.lookahead\npickle lookahead","url":"pickle.html#lookahead"},{"doc":"pickle","title":"lowercase_ascii_letter","content":"pub fn lowercase_ascii_letter(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a lowercase ASCII letter.\nSynonyms:\npickle.lowercase_ascii_letter\npickle lowercase_ascii_letter","url":"pickle.html#lowercase_ascii_letter"},{"doc":"pickle","title":"many","content":"pub fn many(\n  initial_value: a,\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n  acc: fn(d, b) -&gt; d,\n) -&gt; fn(Parsed(d)) -&gt; Result(Parsed(d), ParserFailure(c))\n Applies the initial value to the given parser zero to `n`\n times until it fails.\nSynonyms:\npickle.many\npickle many","url":"pickle.html#many"},{"doc":"pickle","title":"many1","content":"pub fn many1(\n  initial_value: a,\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n  acc: fn(d, b) -&gt; d,\n) -&gt; fn(Parsed(d)) -&gt; Result(Parsed(d), ParserFailure(c))\n Applies the initial value to the given parser one to `n`\n times until it fails.\n \n The given parser must succeed at least once.\nSynonyms:\npickle.many1\npickle many1","url":"pickle.html#many1"},{"doc":"pickle","title":"map","content":"pub fn map(\n  mapper: fn(a) -&gt; b,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c))\n Maps the value of the parser.\nSynonyms:\npickle.map\npickle map","url":"pickle.html#map"},{"doc":"pickle","title":"map_error","content":"pub fn map_error(\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b)),\n  mapper: fn(ParserFailure(b)) -&gt; b,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Maps the error returned by the parser.\nSynonyms:\npickle.map_error\npickle map_error","url":"pickle.html#map_error"},{"doc":"pickle","title":"not","content":"pub fn not(\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n  error: c,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(c))\n Succeeds and backtracks if the given parser fails.\n \n The `error` parameter is meant to convey more information\n to consumers and its value is wrapped in a `NotError`.\nSynonyms:\npickle.not\npickle not","url":"pickle.html#not"},{"doc":"pickle","title":"octal_integer","content":"pub fn octal_integer(\n  mapper: fn(a, Int) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses an octal integer.\nSynonyms:\npickle.octal_integer\npickle octal_integer","url":"pickle.html#octal_integer"},{"doc":"pickle","title":"one_of","content":"pub fn one_of(\n  parsers: List(\n    fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b)),\n  ),\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Applies each given parser in order until one succeeds. If all parsers\n failed, the collected failures wrapped in an `OneOfError` will be\n returned.\nSynonyms:\npickle.one_of\npickle one_of","url":"pickle.html#one_of"},{"doc":"pickle","title":"optional","content":"pub fn optional(\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b)),\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Applies the given parser, and if it fails, ignores its failure and\n backtracks.\nSynonyms:\npickle.optional\npickle optional","url":"pickle.html#optional"},{"doc":"pickle","title":"parse","content":"pub fn parse(\n  input: String,\n  initial_value: a,\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n) -&gt; Result(b, ParserFailure(c))\n Applies the provided input and initial value to the given parser\n to parse the input and transform the initial value.\nSynonyms:\npickle.parse\npickle parse","url":"pickle.html#parse"},{"doc":"pickle","title":"prepend_to_list","content":"pub fn prepend_to_list(value: List(a), appendage: a) -&gt; List(a)\n A mapper to prepend the parsed value of the child parser to\n the list value of the parent parser.\nSynonyms:\npickle.prepend_to_list\npickle prepend_to_list","url":"pickle.html#prepend_to_list"},{"doc":"pickle","title":"return","content":"pub fn return(\n  value: a,\n) -&gt; fn(Parsed(b)) -&gt; Result(Parsed(a), ParserFailure(c))\n Replaces the value of the parser with the given value.\nSynonyms:\npickle.return\npickle return","url":"pickle.html#return"},{"doc":"pickle","title":"skip_until","content":"pub fn skip_until(\n  terminator: fn(Parsed(a)) -&gt;\n    Result(Parsed(b), ParserFailure(c)),\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(c))\n Applies the given parser zero to `n` times until the given terminator\n succeeds and drops the parsed tokens.\n \n It fails if the given terminator doesn&#39;t succeed before no further\n tokens are left to parse.\nSynonyms:\npickle.skip_until\npickle skip_until","url":"pickle.html#skip_until"},{"doc":"pickle","title":"skip_until1","content":"pub fn skip_until1(\n  terminator: fn(Parsed(a)) -&gt;\n    Result(Parsed(b), ParserFailure(c)),\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(c))\n Applies the given parser one to `n` times until the given terminator\n succeeds and drops the parsed tokens.\n \n It fails if no single token could be skipped or the given terminator\n doesn&#39;t succeed before no further tokens are left to parse.\n \n If the terminator succeeds before no single token could be skipped,\n an `Until1Error` with the current parser position will be returned.\nSynonyms:\npickle.skip_until1\npickle skip_until1","url":"pickle.html#skip_until1"},{"doc":"pickle","title":"skip_whitespace","content":"pub fn skip_whitespace() -&gt; fn(Parsed(a)) -&gt;\n  Result(Parsed(a), ParserFailure(b))\n Parses whitespace zero to `n` times until encountering a non-whitespace\n token and drops the parsed whitespace.\nSynonyms:\npickle.skip_whitespace\npickle skip_whitespace","url":"pickle.html#skip_whitespace"},{"doc":"pickle","title":"skip_whitespace1","content":"pub fn skip_whitespace1() -&gt; fn(Parsed(a)) -&gt;\n  Result(Parsed(a), ParserFailure(b))\n Parses whitespace one to `n` times until encountering a non-whitespace\n token and drops the parsed whitespace.\n \n It fails if not at least one whitespace token could be parsed.\nSynonyms:\npickle.skip_whitespace1\npickle skip_whitespace1","url":"pickle.html#skip_whitespace1"},{"doc":"pickle","title":"string","content":"pub fn string(\n  expected: String,\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses a specific string.\nSynonyms:\npickle.string\npickle string","url":"pickle.html#string"},{"doc":"pickle","title":"then","content":"pub fn then(\n  prev: fn(Parsed(a)) -&gt; Result(Parsed(b), ParserFailure(c)),\n  successor: fn(Parsed(b)) -&gt; Result(Parsed(d), ParserFailure(c)),\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(d), ParserFailure(c))\n Chains two given parsers.\n \n If `prev` fails, `successor` won&#39;t be invoked.\nSynonyms:\npickle.then\npickle then","url":"pickle.html#then"},{"doc":"pickle","title":"until","content":"pub fn until(\n  initial_value: a,\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b)),\n  terminator: fn(Parsed(c)) -&gt;\n    Result(Parsed(d), ParserFailure(b)),\n  acc: fn(c, a) -&gt; c,\n) -&gt; fn(Parsed(c)) -&gt; Result(Parsed(c), ParserFailure(b))\n Applies the given parser zero to `n` times until the given terminator\n succeeds.\n \n It fails if the given parser fails or the given terminator doesn&#39;t\n succeed before no further tokens are left to parse.\nSynonyms:\npickle.until\npickle until","url":"pickle.html#until"},{"doc":"pickle","title":"until1","content":"pub fn until1(\n  initial_value: a,\n  parser: fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b)),\n  terminator: fn(Parsed(c)) -&gt;\n    Result(Parsed(d), ParserFailure(b)),\n  acc: fn(c, a) -&gt; c,\n) -&gt; fn(Parsed(c)) -&gt; Result(Parsed(c), ParserFailure(b))\n Applies the given parser one to `n` times until the given terminator\n succeeds.\n \n It fails if the given parser fails or could not be applied once, or the given\n terminator doesn&#39;t succeed before no further tokens are left to parse.\n \n If the terminator succeeds before the given parser could succeed, an\n `Until1Error` with the current parser position will be returned.\nSynonyms:\npickle.until1\npickle until1","url":"pickle.html#until1"},{"doc":"pickle","title":"uppercase_ascii_letter","content":"pub fn uppercase_ascii_letter(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses an uppercase ASCII letter.\nSynonyms:\npickle.uppercase_ascii_letter\npickle uppercase_ascii_letter","url":"pickle.html#uppercase_ascii_letter"},{"doc":"pickle","title":"whitespace","content":"pub fn whitespace(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses whitespace zero to `n` times until encountering a non-whitespace\n token.\nSynonyms:\npickle.whitespace\npickle whitespace","url":"pickle.html#whitespace"},{"doc":"pickle","title":"whitespace1","content":"pub fn whitespace1(\n  mapper: fn(a, String) -&gt; a,\n) -&gt; fn(Parsed(a)) -&gt; Result(Parsed(a), ParserFailure(b))\n Parses whitespace one to `n` times until encountering a non-whitespace\n token.\n \n It fails if not at least one whitespace token could be parsed.\nSynonyms:\npickle.whitespace1\npickle whitespace1","url":"pickle.html#whitespace1"},{"doc":"pickle","title":"pickle","content":"","url":"pickle.html"}]);